@@ -1,8 +1,11 @@
 import abc
 import copy
-import re
 import logging
+import re
+import sys
+import importlib
 from functools import wraps
+from typing import List, Type
 
 try:
     import rpy2
@@ -13,12 +16,6 @@ from packaging import version
 
 
 logger = logging.getLogger(__name__)
-SHELL_COMMAND_RE = re.compile(r"^!", flags=re.M)
-COMMENTED_SHELL_COMMAND_RE = re.compile(r"^# !#", flags=re.M)
-MAGIC_COMMAND_RE = re.compile(r"^%", flags=re.M)
-COMMENTED_MAGIC_COMMAND_RE = re.compile(r"^# %#", flags=re.M)
-IPYTHON_HELP_RE = re.compile(r"(^(?!\s*#)(.*)\?\??)$", flags=re.M)
-COMMENTED_IPYTHON_HELP_RE = re.compile(r"# \?#", flags=re.M)
 
 
 INCOMPATIBLE_MAGIC_LANGUAGES = [
@@ -39,7 +36,8 @@ INCOMPATIBLE_MAGIC_LANGUAGES = [
     "configure",
     "logs",
     "sql",
-    "local"
+    "local",
+    "sparksql",
 ]
 
 
@@ -59,6 +57,118 @@ class BaseFormatter(abc.ABC):
         pass
 
 
+class BaseLineEscaper(abc.ABC):
+    """A base class for defining how to escape certain sequence of text to avoid formatting."""
+
+    def __init__(self, code: str) -> None:
+        self.code = code
+
+    @property
+    @abc.abstractmethod
+    def langs(self) -> List[str]:
+        return
+
+    @abc.abstractmethod
+    def escape(self, line: str) -> str:
+        pass
+
+    @abc.abstractmethod
+    def unescape(self, line: str) -> str:
+        pass
+
+
+class MagicCommandEscaper(BaseLineEscaper):
+    langs = ["python"]
+    escaped_line_start = "# \x01 "
+    unesacpe_start = len(escaped_line_start)
+
+    def escape(self, line: str) -> str:
+        if line.lstrip().startswith("%"):
+            line = f"{self.escaped_line_start}{line}"
+        return line
+
+    def unescape(self, line: str) -> str:
+        if line.lstrip().startswith(self.escaped_line_start):
+            line = line[self.unesacpe_start :]
+        return line
+
+
+class RunScriptEscaper(BaseLineEscaper):
+    langs = ["python"]
+    escaped_line_start = "# \x01 "
+    unesacpe_start = len(escaped_line_start)
+
+    def escape(self, line: str) -> str:
+        if re.match(pattern="run\s+\w+", string=line.lstrip()):
+            line = f"{self.escaped_line_start}{line}"
+        return line
+
+    def unescape(self, line: str) -> str:
+        if line.lstrip().startswith(self.escaped_line_start):
+            line = line[self.unesacpe_start :]
+        return line
+
+
+class HelpEscaper(BaseLineEscaper):
+
+    langs = ["python"]
+    escaped_line_start = "# \x01 "
+    unesacpe_start = len(escaped_line_start)
+
+    def escape(self, line: str) -> str:
+        if (line.endswith("??") or line.endswith("?")) and "#" not in line:
+            line = f"{self.escaped_line_start}{line}"
+        return line
+
+    def unescape(self, line: str) -> str:
+        if line.lstrip().startswith(self.escaped_line_start):
+            line = line[self.unesacpe_start :]
+        return line
+
+
+class CommandEscaper(BaseLineEscaper):
+
+    langs = ["python"]
+    escaped_line_start = "# \x01 "
+    unesacpe_start = len(escaped_line_start)
+
+    def escape(self, line: str) -> str:
+        if line.lstrip().startswith("!"):
+            line = f"{self.escaped_line_start}{line}"
+        return line
+
+    def unescape(self, line: str) -> str:
+        if line.lstrip().startswith(self.escaped_line_start):
+            line = line[self.unesacpe_start :]
+        return line
+
+
+class QuartoCommentEscaper(BaseLineEscaper):
+
+    langs = ["python"]
+    escaped_line_start = "# \x01 "
+    unesacpe_start = len(escaped_line_start)
+
+    def escape(self, line: str) -> str:
+        if line.lstrip().startswith("#| "):
+            line = f"{self.escaped_line_start}{line}"
+        return line
+
+    def unescape(self, line: str) -> str:
+        if line.lstrip().startswith(self.escaped_line_start):
+            line = line[self.unesacpe_start :]
+        return line
+
+
+ESCAPER_CLASSES: List[Type[BaseLineEscaper]] = [
+    MagicCommandEscaper,
+    HelpEscaper,
+    CommandEscaper,
+    QuartoCommentEscaper,
+    RunScriptEscaper,
+]
+
+
 def handle_line_ending_and_magic(func):
     @wraps(func)
     def wrapped(self, code: str, notebook: bool, **options) -> str:
@@ -70,13 +180,19 @@ def handle_line_ending_and_magic(func):
 
         has_semicolon = code.strip().endswith(";")
 
-        code = re.sub(MAGIC_COMMAND_RE, "# %#", code)
-        code = re.sub(SHELL_COMMAND_RE, "# !#", code)
-        code = re.sub(IPYTHON_HELP_RE, r"# ?#\1", code)
+        escapers = [escaper_cls(code) for escaper_cls in ESCAPER_CLASSES]
+
+        lines = code.splitlines()
+        for escaper in escapers:
+            lines = map(escaper.escape, lines)
+        code = "\n".join(lines)
+
         code = func(self, code, notebook, **options)
-        code = re.sub(COMMENTED_MAGIC_COMMAND_RE, "%", code)
-        code = re.sub(COMMENTED_SHELL_COMMAND_RE, "!", code)
-        code = re.sub(COMMENTED_IPYTHON_HELP_RE, "", code)
+
+        lines = code.splitlines()
+        for escaper in escapers:
+            lines = map(escaper.unescape, lines)
+        code = "\n".join(lines)
 
         if notebook:
             code = code.rstrip()
@@ -88,14 +204,46 @@ def handle_line_ending_and_magic(func):
     return wrapped
 
 
-class BlackFormatter(BaseFormatter):
+BLUE_MONKEY_PATCHED = False
 
-    label = "Apply Black Formatter"
+
+def import_black():
+    global BLUE_MONKEY_PATCHED
+    if BLUE_MONKEY_PATCHED:
+        for module in list(sys.modules):
+            if module.startswith("black."):
+                importlib.reload(sys.modules[module])
+
+        import black
+
+        black = importlib.reload(black)
+        BLUE_MONKEY_PATCHED = False
+    else:
+        import black
+
+    return black
+
+
+def import_blue():
+    """Import blue and perform monkey patch."""
+    global BLUE_MONKEY_PATCHED
+    import blue
+
+    if not BLUE_MONKEY_PATCHED:
+        blue.monkey_patch_black(blue.Mode.synchronous)
+        BLUE_MONKEY_PATCHED = True
+
+    return blue
+
+
+class BlueFormatter(BaseFormatter):
+
+    label = "Apply Blue Formatter"
 
     @property
     def importable(self) -> bool:
         try:
-            import black
+            import_blue()
 
             return True
         except ImportError:
@@ -103,7 +251,34 @@ class BlackFormatter(BaseFormatter):
 
     @staticmethod
     def handle_options(**options):
-        import black
+        blue = import_blue()
+
+        return {"mode": blue.black.FileMode(**options)}
+
+    @handle_line_ending_and_magic
+    def format_code(self, code: str, notebook: bool, **options) -> str:
+        blue = import_blue()
+
+        code = blue.black.format_str(code, **self.handle_options(**options))
+        return code
+
+
+class BlackFormatter(BaseFormatter):
+
+    label = "Apply Black Formatter"
+
+    @property
+    def importable(self) -> bool:
+        try:
+            import_black()
+
+            return True
+        except ImportError:
+            return False
+
+    @staticmethod
+    def handle_options(**options):
+        black = import_black()
 
         file_mode_change_version = version.parse("19.3b0")
         current_black_version = version.parse(black.__version__)
@@ -114,7 +289,7 @@ class BlackFormatter(BaseFormatter):
 
     @handle_line_ending_and_magic
     def format_code(self, code: str, notebook: bool, **options) -> str:
-        import black
+        black = import_black()
 
         code = black.format_str(code, **self.handle_options(**options))
         return code
@@ -266,6 +441,7 @@ class StylerFormatter(BaseFormatter):
 
 SERVER_FORMATTERS = {
     "black": BlackFormatter(),
+    "blue": BlueFormatter(),
     "autopep8": Autopep8Formatter(),
     "yapf": YapfFormatter(),
     "isort": IsortFormatter(),
